# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Utterance(UniversalBaseModel):
    """
    A single utterance for topic segmentation.
    """

    text: str = pydantic.Field()
    """
    Utterance text content
    """

    start_ms: int = pydantic.Field()
    """
    Start time in milliseconds
    """

    end_ms: int = pydantic.Field()
    """
    End time in milliseconds
    """

    speaker: typing.Optional[str] = pydantic.Field(default=None)
    """
    Speaker label
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ImageUploadResponse(UniversalBaseModel):
    """
    Response schema for image upload.
    """

    success: bool = pydantic.Field()
    """
    Whether the upload was successful
    """

    message: str = pydantic.Field()
    """
    Success message
    """

    urls: typing.Dict[str, str] = pydantic.Field()
    """
    Image URLs for different sizes (thumbnail, medium, full)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ComposioSyncResponse(UniversalBaseModel):
    """
    Response schema for connector sync endpoint.
    """

    job_id: str
    status: str
    tools_synced: int
    tools_created: int
    tools_updated: int
    tools_unchanged: typing.Optional[int] = None
    sync_mode: typing.Optional[str] = None
    message: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UserProfileResponse(UniversalBaseModel):
    """
    Schema for user profile response with detailed information.
    """

    id: str = pydantic.Field()
    """
    User ID with usr_ prefix
    """

    email: str
    username: typing.Optional[str] = None
    full_name: typing.Optional[str] = None
    first_name: typing.Optional[str] = None
    last_name: typing.Optional[str] = None
    birthday: typing.Optional[dt.date] = None
    profile_image: typing.Optional[str] = None
    timezone: typing.Optional[str] = None
    country_id: typing.Optional[str] = None
    post_code: typing.Optional[str] = None
    gender: typing.Optional[str] = None
    is_active: bool
    last_verified: typing.Optional[dt.datetime] = None
    last_login: typing.Optional[dt.datetime] = None
    created_at: dt.datetime
    updated_at: dt.datetime
    current_organization_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AccessOverviewGrant(UniversalBaseModel):
    """
    Grant entry in the access overview.
    """

    id: str
    entity_type: str
    entity_id: str
    grantee_type: str
    grantee_id: str
    access_level: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.device_information import DeviceInformation
from ..types.email_validation_response import EmailValidationResponse
from ..types.google_authorize_response import GoogleAuthorizeResponse
from ..types.login_response import LoginResponse
from ..types.login_verification_response import LoginVerificationResponse
from ..types.logout_response import LogoutResponse
from ..types.password_reset_response import PasswordResetResponse
from ..types.password_reset_verify_response import PasswordResetVerifyResponse
from ..types.password_update_response import PasswordUpdateResponse
from ..types.refresh_token_response import RefreshTokenResponse
from ..types.register_response import RegisterResponse
from ..types.resend_code_response import ResendCodeResponse
from ..types.token_validation_response import TokenValidationResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAuthenticationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register(
        self,
        *,
        email: str,
        password: str,
        full_name: str,
        user_name: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RegisterResponse]:
        """
        Register a new user account with email verification.

        Creates a new user account, sends a 4-digit verification code via email,
        and creates an EmailVerification record. User must verify their email
        using the /verify endpoint to complete registration.

        Parameters
        ----------
        email : str
            User email address (primary identifier)

        password : str
            Password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        full_name : str
            User's full name

        user_name : typing.Optional[str]
            Preferred username (defaults to email prefix if omitted, min 3 chars)

        organization_id : typing.Optional[str]
            Organization UID to associate the user with

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RegisterResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/register",
            method="POST",
            json={
                "email": email,
                "password": password,
                "full_name": full_name,
                "user_name": user_name,
                "organization_id": organization_id,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RegisterResponse,
                    parse_obj_as(
                        type_=RegisterResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LoginResponse]:
        """
        Authenticate user credentials and initiate two-factor authentication (2FA) via email verification.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LoginResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/login",
            method="POST",
            json={
                "email_or_username": email_or_username,
                "password": password,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LoginResponse,
                    parse_obj_as(
                        type_=LoginResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def verify_email(
        self,
        *,
        email_key: str,
        verification_code: int,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LoginVerificationResponse]:
        """
        Verify email verification code and return JWT tokens.

        Parameters
        ----------
        email_key : str
            Email verification key

        verification_code : int
            4-digit verification code

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LoginVerificationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/verify",
            method="POST",
            json={
                "email_key": email_key,
                "verification_code": verification_code,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LoginVerificationResponse,
                    parse_obj_as(
                        type_=LoginVerificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def refresh_token(
        self, *, refresh_token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RefreshTokenResponse]:
        """
        Refresh an expired access token using a valid refresh token.

        Parameters
        ----------
        refresh_token : str
            Refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RefreshTokenResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/refresh",
            method="POST",
            json={
                "refresh_token": refresh_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RefreshTokenResponse,
                    parse_obj_as(
                        type_=RefreshTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def logout(
        self, *, refresh_token: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[LogoutResponse]:
        """
        Logout the current user and invalidate their authentication tokens.

        Parameters
        ----------
        refresh_token : typing.Optional[str]
            Refresh token to invalidate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LogoutResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/logout",
            method="POST",
            json={
                "refresh_token": refresh_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LogoutResponse,
                    parse_obj_as(
                        type_=LogoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def request_password_reset(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PasswordResetResponse]:
        """
        Request a password reset for a user account.

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset/verify
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PasswordResetResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/password/reset",
            method="POST",
            json={
                "email": email,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordResetResponse,
                    parse_obj_as(
                        type_=PasswordResetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def verify_password_reset(
        self,
        *,
        email: str,
        verification_code: int,
        new_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PasswordResetVerifyResponse]:
        """
        Verify password reset code and set new password.

        Validates the 4-digit code and updates the user's password.
        All existing refresh tokens are revoked, forcing re-authentication.

        **Security Features:**
        - Code expires after 5 minutes
        - One-time use codes (marked as used after verification)
        - Password strength validation
        - All sessions revoked on success

        **Errors:**
        - 404: User not found
        - 422: Invalid/expired code or weak password

        Parameters
        ----------
        email : str
            User email address

        verification_code : int
            4-digit verification code

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PasswordResetVerifyResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/password/reset/verify",
            method="POST",
            json={
                "email": email,
                "verification_code": verification_code,
                "new_password": new_password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordResetVerifyResponse,
                    parse_obj_as(
                        type_=PasswordResetVerifyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_password(
        self,
        *,
        current_password: str,
        new_password: str,
        confirm_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PasswordUpdateResponse]:
        """
        Update password for authenticated user.

        Requires current password verification and enforces password strength requirements.
        All other device sessions are revoked while keeping the current session active.

        **Security Features:**
        - Requires authentication (Bearer token)
        - Verifies current password before allowing change
        - Password strength validation
        - Keeps current session active
        - Revokes all other device sessions
        - Logs password change for audit

        **Password Requirements:**
        - Minimum 8 characters
        - At least one special character
        - Not in common weak passwords list

        **Errors:**
        - 401: Current password is incorrect
        - 404: User not found
        - 422: Passwords don't match or new password is weak

        Parameters
        ----------
        current_password : str
            Current password for verification

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        confirm_password : str
            Confirmation of new password (must match new_password)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PasswordUpdateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/password/update",
            method="PUT",
            json={
                "current_password": current_password,
                "new_password": new_password,
                "confirm_password": confirm_password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordUpdateResponse,
                    parse_obj_as(
                        type_=PasswordUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def validate_email(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EmailValidationResponse]:
        """
        Validate email for registration.

        Checks if email is:
        - Valid format
        - Not already registered
        - Associated with an organization domain

        This endpoint is used during registration to provide real-time feedback
        about email validity and availability.

        **Returns:**
        - valid: Whether email format is valid
        - available: Whether email is available for registration
        - organization_id: Organization ID if email domain matches (optional)
        - organization_name: Organization name if email domain matches (optional)

        Parameters
        ----------
        email : str
            Email address to validate for signup eligibility

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EmailValidationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/validate-email",
            method="POST",
            json={
                "email": email,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EmailValidationResponse,
                    parse_obj_as(
                        type_=EmailValidationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def resend_verification_code(
        self, *, email_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ResendCodeResponse]:
        """
        Resend verification code.

        Generates and sends a new 4-digit verification code to the user's email.
        Works for login, registration, and password reset verifications.

        **Security Features:**
        - Validates verification record exists
        - Checks if verification already used
        - Generates new random code
        - Updates expiry time

        **Errors:**
        - 404: Verification record not found
        - 422: Verification already used
        - 500: Email sending failed

        Parameters
        ----------
        email_key : str
            Email verification key

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResendCodeResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/resend-code",
            method="POST",
            json={
                "email_key": email_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResendCodeResponse,
                    parse_obj_as(
                        type_=ResendCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def forgot_password(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PasswordResetResponse]:
        """
        Request a password reset (alias for /password/reset).

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PasswordResetResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/password/forgot",
            method="POST",
            json={
                "email": email,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordResetResponse,
                    parse_obj_as(
                        type_=PasswordResetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def validate_token(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TokenValidationResponse]:
        """
        Validate current authentication token.

        Used by frontend to check if the session is still valid.
        If the request reaches this endpoint (passes AuthContext dependency),
        the token is valid.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TokenValidationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/validate",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TokenValidationResponse,
                    parse_obj_as(
                        type_=TokenValidationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def dev_login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LoginVerificationResponse]:
        """
        Development login (bypass email verification).

        **WARNING:** This endpoint is only available in non-production environments
        or when explicitly enabled via configuration.

        It validates credentials and immediately returns authentication tokens
        without sending a verification code.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LoginVerificationResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/dev-login",
            method="POST",
            json={
                "email_or_username": email_or_username,
                "password": password,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LoginVerificationResponse,
                    parse_obj_as(
                        type_=LoginVerificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def google_authorize(
        self,
        *,
        redirect_uri: str,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GoogleAuthorizeResponse]:
        """
        Generate Google OAuth authorization URL for Sign-In.

        Parameters
        ----------
        redirect_uri : str
            Frontend URL to redirect to after authentication

        organization_id : typing.Optional[str]
            Optional organization ID to associate user with

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GoogleAuthorizeResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/google/authorize",
            method="POST",
            json={
                "redirect_uri": redirect_uri,
                "organization_id": organization_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GoogleAuthorizeResponse,
                    parse_obj_as(
                        type_=GoogleAuthorizeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def google_callback(
        self,
        *,
        code: str,
        state: str,
        error: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Handle Google OAuth callback and show success/error page.

        Parameters
        ----------
        code : str
            Authorization code from Google

        state : str
            CSRF state token

        error : typing.Optional[str]
            Error from Google

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/google/callback",
            method="GET",
            params={
                "code": code,
                "state": state,
                "error": error,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAuthenticationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register(
        self,
        *,
        email: str,
        password: str,
        full_name: str,
        user_name: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RegisterResponse]:
        """
        Register a new user account with email verification.

        Creates a new user account, sends a 4-digit verification code via email,
        and creates an EmailVerification record. User must verify their email
        using the /verify endpoint to complete registration.

        Parameters
        ----------
        email : str
            User email address (primary identifier)

        password : str
            Password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        full_name : str
            User's full name

        user_name : typing.Optional[str]
            Preferred username (defaults to email prefix if omitted, min 3 chars)

        organization_id : typing.Optional[str]
            Organization UID to associate the user with

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RegisterResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/register",
            method="POST",
            json={
                "email": email,
                "password": password,
                "full_name": full_name,
                "user_name": user_name,
                "organization_id": organization_id,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RegisterResponse,
                    parse_obj_as(
                        type_=RegisterResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LoginResponse]:
        """
        Authenticate user credentials and initiate two-factor authentication (2FA) via email verification.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LoginResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/login",
            method="POST",
            json={
                "email_or_username": email_or_username,
                "password": password,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LoginResponse,
                    parse_obj_as(
                        type_=LoginResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def verify_email(
        self,
        *,
        email_key: str,
        verification_code: int,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LoginVerificationResponse]:
        """
        Verify email verification code and return JWT tokens.

        Parameters
        ----------
        email_key : str
            Email verification key

        verification_code : int
            4-digit verification code

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LoginVerificationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/verify",
            method="POST",
            json={
                "email_key": email_key,
                "verification_code": verification_code,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LoginVerificationResponse,
                    parse_obj_as(
                        type_=LoginVerificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def refresh_token(
        self, *, refresh_token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RefreshTokenResponse]:
        """
        Refresh an expired access token using a valid refresh token.

        Parameters
        ----------
        refresh_token : str
            Refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RefreshTokenResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/refresh",
            method="POST",
            json={
                "refresh_token": refresh_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RefreshTokenResponse,
                    parse_obj_as(
                        type_=RefreshTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def logout(
        self, *, refresh_token: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[LogoutResponse]:
        """
        Logout the current user and invalidate their authentication tokens.

        Parameters
        ----------
        refresh_token : typing.Optional[str]
            Refresh token to invalidate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LogoutResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/logout",
            method="POST",
            json={
                "refresh_token": refresh_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LogoutResponse,
                    parse_obj_as(
                        type_=LogoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def request_password_reset(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PasswordResetResponse]:
        """
        Request a password reset for a user account.

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset/verify
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PasswordResetResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/password/reset",
            method="POST",
            json={
                "email": email,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordResetResponse,
                    parse_obj_as(
                        type_=PasswordResetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def verify_password_reset(
        self,
        *,
        email: str,
        verification_code: int,
        new_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PasswordResetVerifyResponse]:
        """
        Verify password reset code and set new password.

        Validates the 4-digit code and updates the user's password.
        All existing refresh tokens are revoked, forcing re-authentication.

        **Security Features:**
        - Code expires after 5 minutes
        - One-time use codes (marked as used after verification)
        - Password strength validation
        - All sessions revoked on success

        **Errors:**
        - 404: User not found
        - 422: Invalid/expired code or weak password

        Parameters
        ----------
        email : str
            User email address

        verification_code : int
            4-digit verification code

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PasswordResetVerifyResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/password/reset/verify",
            method="POST",
            json={
                "email": email,
                "verification_code": verification_code,
                "new_password": new_password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordResetVerifyResponse,
                    parse_obj_as(
                        type_=PasswordResetVerifyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_password(
        self,
        *,
        current_password: str,
        new_password: str,
        confirm_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PasswordUpdateResponse]:
        """
        Update password for authenticated user.

        Requires current password verification and enforces password strength requirements.
        All other device sessions are revoked while keeping the current session active.

        **Security Features:**
        - Requires authentication (Bearer token)
        - Verifies current password before allowing change
        - Password strength validation
        - Keeps current session active
        - Revokes all other device sessions
        - Logs password change for audit

        **Password Requirements:**
        - Minimum 8 characters
        - At least one special character
        - Not in common weak passwords list

        **Errors:**
        - 401: Current password is incorrect
        - 404: User not found
        - 422: Passwords don't match or new password is weak

        Parameters
        ----------
        current_password : str
            Current password for verification

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        confirm_password : str
            Confirmation of new password (must match new_password)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PasswordUpdateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/password/update",
            method="PUT",
            json={
                "current_password": current_password,
                "new_password": new_password,
                "confirm_password": confirm_password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordUpdateResponse,
                    parse_obj_as(
                        type_=PasswordUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def validate_email(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EmailValidationResponse]:
        """
        Validate email for registration.

        Checks if email is:
        - Valid format
        - Not already registered
        - Associated with an organization domain

        This endpoint is used during registration to provide real-time feedback
        about email validity and availability.

        **Returns:**
        - valid: Whether email format is valid
        - available: Whether email is available for registration
        - organization_id: Organization ID if email domain matches (optional)
        - organization_name: Organization name if email domain matches (optional)

        Parameters
        ----------
        email : str
            Email address to validate for signup eligibility

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EmailValidationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/validate-email",
            method="POST",
            json={
                "email": email,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EmailValidationResponse,
                    parse_obj_as(
                        type_=EmailValidationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def resend_verification_code(
        self, *, email_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ResendCodeResponse]:
        """
        Resend verification code.

        Generates and sends a new 4-digit verification code to the user's email.
        Works for login, registration, and password reset verifications.

        **Security Features:**
        - Validates verification record exists
        - Checks if verification already used
        - Generates new random code
        - Updates expiry time

        **Errors:**
        - 404: Verification record not found
        - 422: Verification already used
        - 500: Email sending failed

        Parameters
        ----------
        email_key : str
            Email verification key

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResendCodeResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/resend-code",
            method="POST",
            json={
                "email_key": email_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResendCodeResponse,
                    parse_obj_as(
                        type_=ResendCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def forgot_password(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PasswordResetResponse]:
        """
        Request a password reset (alias for /password/reset).

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PasswordResetResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/password/forgot",
            method="POST",
            json={
                "email": email,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PasswordResetResponse,
                    parse_obj_as(
                        type_=PasswordResetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def validate_token(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TokenValidationResponse]:
        """
        Validate current authentication token.

        Used by frontend to check if the session is still valid.
        If the request reaches this endpoint (passes AuthContext dependency),
        the token is valid.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TokenValidationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/validate",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TokenValidationResponse,
                    parse_obj_as(
                        type_=TokenValidationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def dev_login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LoginVerificationResponse]:
        """
        Development login (bypass email verification).

        **WARNING:** This endpoint is only available in non-production environments
        or when explicitly enabled via configuration.

        It validates credentials and immediately returns authentication tokens
        without sending a verification code.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LoginVerificationResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/dev-login",
            method="POST",
            json={
                "email_or_username": email_or_username,
                "password": password,
                "device_information": convert_and_respect_annotation_metadata(
                    object_=device_information, annotation=typing.Optional[DeviceInformation], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LoginVerificationResponse,
                    parse_obj_as(
                        type_=LoginVerificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def google_authorize(
        self,
        *,
        redirect_uri: str,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GoogleAuthorizeResponse]:
        """
        Generate Google OAuth authorization URL for Sign-In.

        Parameters
        ----------
        redirect_uri : str
            Frontend URL to redirect to after authentication

        organization_id : typing.Optional[str]
            Optional organization ID to associate user with

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GoogleAuthorizeResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/google/authorize",
            method="POST",
            json={
                "redirect_uri": redirect_uri,
                "organization_id": organization_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GoogleAuthorizeResponse,
                    parse_obj_as(
                        type_=GoogleAuthorizeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def google_callback(
        self,
        *,
        code: str,
        state: str,
        error: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Handle Google OAuth callback and show success/error page.

        Parameters
        ----------
        code : str
            Authorization code from Google

        state : str
            CSRF state token

        error : typing.Optional[str]
            Error from Google

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/google/callback",
            method="GET",
            params={
                "code": code,
                "state": state,
                "error": error,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class KnowledgeSliceResponse(UniversalBaseModel):
    """
    Schema for knowledge slice response.
    """

    id: str
    name: str
    description: typing.Optional[str] = None
    type: str
    is_composite: bool
    owner_type: str
    owner_id: str
    status: str
    member_slice_ids: typing.Optional[typing.List[str]] = None
    store_count: int
    organization_id: str
    created_by: typing.Optional[str] = None
    created_at: dt.datetime
    updated_at: dt.datetime
    archived_at: typing.Optional[dt.datetime] = None
    archived_by: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.device_information import DeviceInformation
from ..types.email_validation_response import EmailValidationResponse
from ..types.google_authorize_response import GoogleAuthorizeResponse
from ..types.login_response import LoginResponse
from ..types.login_verification_response import LoginVerificationResponse
from ..types.logout_response import LogoutResponse
from ..types.password_reset_response import PasswordResetResponse
from ..types.password_reset_verify_response import PasswordResetVerifyResponse
from ..types.password_update_response import PasswordUpdateResponse
from ..types.refresh_token_response import RefreshTokenResponse
from ..types.register_response import RegisterResponse
from ..types.resend_code_response import ResendCodeResponse
from ..types.token_validation_response import TokenValidationResponse
from .raw_client import AsyncRawAuthenticationClient, RawAuthenticationClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AuthenticationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthenticationClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuthenticationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthenticationClient
        """
        return self._raw_client

    def register(
        self,
        *,
        email: str,
        password: str,
        full_name: str,
        user_name: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterResponse:
        """
        Register a new user account with email verification.

        Creates a new user account, sends a 4-digit verification code via email,
        and creates an EmailVerification record. User must verify their email
        using the /verify endpoint to complete registration.

        Parameters
        ----------
        email : str
            User email address (primary identifier)

        password : str
            Password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        full_name : str
            User's full name

        user_name : typing.Optional[str]
            Preferred username (defaults to email prefix if omitted, min 3 chars)

        organization_id : typing.Optional[str]
            Organization UID to associate the user with

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos, DeviceInformation

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.register(
            email="user@company.com",
            password="SecurePassword123!",
            full_name="John Doe",
            user_name="johndoe",
            organization_id="org_12345678901234567890123456789012",
            device_information=DeviceInformation(
                device="Chrome Browser",
                device_id="device-123",
                platform="web",
            ),
        )
        """
        _response = self._raw_client.register(
            email=email,
            password=password,
            full_name=full_name,
            user_name=user_name,
            organization_id=organization_id,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    def login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LoginResponse:
        """
        Authenticate user credentials and initiate two-factor authentication (2FA) via email verification.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LoginResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos, DeviceInformation

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.login(
            email_or_username="user@example.com",
            password="SecurePassword123!",
            device_information=DeviceInformation(
                device="Chrome Browser",
                device_id="device-123",
                operating_system="macOS",
                platform="web",
            ),
        )
        """
        _response = self._raw_client.login(
            email_or_username=email_or_username,
            password=password,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    def verify_email(
        self,
        *,
        email_key: str,
        verification_code: int,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LoginVerificationResponse:
        """
        Verify email verification code and return JWT tokens.

        Parameters
        ----------
        email_key : str
            Email verification key

        verification_code : int
            4-digit verification code

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LoginVerificationResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos, DeviceInformation

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.verify_email(
            email_key="uuid-12345678-1234-1234-1234-123456789abc",
            verification_code=1234,
            device_information=DeviceInformation(
                device_id="device-123",
                platform="web",
            ),
        )
        """
        _response = self._raw_client.verify_email(
            email_key=email_key,
            verification_code=verification_code,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    def refresh_token(
        self, *, refresh_token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RefreshTokenResponse:
        """
        Refresh an expired access token using a valid refresh token.

        Parameters
        ----------
        refresh_token : str
            Refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefreshTokenResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.refresh_token(
            refresh_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...",
        )
        """
        _response = self._raw_client.refresh_token(refresh_token=refresh_token, request_options=request_options)
        return _response.data

    def logout(
        self, *, refresh_token: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> LogoutResponse:
        """
        Logout the current user and invalidate their authentication tokens.

        Parameters
        ----------
        refresh_token : typing.Optional[str]
            Refresh token to invalidate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LogoutResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.logout(
            refresh_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...",
        )
        """
        _response = self._raw_client.logout(refresh_token=refresh_token, request_options=request_options)
        return _response.data

    def request_password_reset(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PasswordResetResponse:
        """
        Request a password reset for a user account.

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset/verify
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordResetResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.request_password_reset(
            email="user@example.com",
        )
        """
        _response = self._raw_client.request_password_reset(email=email, request_options=request_options)
        return _response.data

    def verify_password_reset(
        self,
        *,
        email: str,
        verification_code: int,
        new_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PasswordResetVerifyResponse:
        """
        Verify password reset code and set new password.

        Validates the 4-digit code and updates the user's password.
        All existing refresh tokens are revoked, forcing re-authentication.

        **Security Features:**
        - Code expires after 5 minutes
        - One-time use codes (marked as used after verification)
        - Password strength validation
        - All sessions revoked on success

        **Errors:**
        - 404: User not found
        - 422: Invalid/expired code or weak password

        Parameters
        ----------
        email : str
            User email address

        verification_code : int
            4-digit verification code

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordResetVerifyResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.verify_password_reset(
            email="user@example.com",
            verification_code=1234,
            new_password="NewSecurePassword123!",
        )
        """
        _response = self._raw_client.verify_password_reset(
            email=email, verification_code=verification_code, new_password=new_password, request_options=request_options
        )
        return _response.data

    def update_password(
        self,
        *,
        current_password: str,
        new_password: str,
        confirm_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PasswordUpdateResponse:
        """
        Update password for authenticated user.

        Requires current password verification and enforces password strength requirements.
        All other device sessions are revoked while keeping the current session active.

        **Security Features:**
        - Requires authentication (Bearer token)
        - Verifies current password before allowing change
        - Password strength validation
        - Keeps current session active
        - Revokes all other device sessions
        - Logs password change for audit

        **Password Requirements:**
        - Minimum 8 characters
        - At least one special character
        - Not in common weak passwords list

        **Errors:**
        - 401: Current password is incorrect
        - 404: User not found
        - 422: Passwords don't match or new password is weak

        Parameters
        ----------
        current_password : str
            Current password for verification

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        confirm_password : str
            Confirmation of new password (must match new_password)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordUpdateResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.update_password(
            current_password="CurrentPassword123!",
            new_password="NewSecurePassword456!",
            confirm_password="NewSecurePassword456!",
        )
        """
        _response = self._raw_client.update_password(
            current_password=current_password,
            new_password=new_password,
            confirm_password=confirm_password,
            request_options=request_options,
        )
        return _response.data

    def validate_email(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> EmailValidationResponse:
        """
        Validate email for registration.

        Checks if email is:
        - Valid format
        - Not already registered
        - Associated with an organization domain

        This endpoint is used during registration to provide real-time feedback
        about email validity and availability.

        **Returns:**
        - valid: Whether email format is valid
        - available: Whether email is available for registration
        - organization_id: Organization ID if email domain matches (optional)
        - organization_name: Organization name if email domain matches (optional)

        Parameters
        ----------
        email : str
            Email address to validate for signup eligibility

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailValidationResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.validate_email(
            email="user@aitronos.com",
        )
        """
        _response = self._raw_client.validate_email(email=email, request_options=request_options)
        return _response.data

    def resend_verification_code(
        self, *, email_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResendCodeResponse:
        """
        Resend verification code.

        Generates and sends a new 4-digit verification code to the user's email.
        Works for login, registration, and password reset verifications.

        **Security Features:**
        - Validates verification record exists
        - Checks if verification already used
        - Generates new random code
        - Updates expiry time

        **Errors:**
        - 404: Verification record not found
        - 422: Verification already used
        - 500: Email sending failed

        Parameters
        ----------
        email_key : str
            Email verification key

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResendCodeResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.resend_verification_code(
            email_key="uuid-12345678-1234-1234-1234-123456789abc",
        )
        """
        _response = self._raw_client.resend_verification_code(email_key=email_key, request_options=request_options)
        return _response.data

    def forgot_password(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PasswordResetResponse:
        """
        Request a password reset (alias for /password/reset).

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordResetResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.forgot_password(
            email="user@example.com",
        )
        """
        _response = self._raw_client.forgot_password(email=email, request_options=request_options)
        return _response.data

    def validate_token(self, *, request_options: typing.Optional[RequestOptions] = None) -> TokenValidationResponse:
        """
        Validate current authentication token.

        Used by frontend to check if the session is still valid.
        If the request reaches this endpoint (passes AuthContext dependency),
        the token is valid.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TokenValidationResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.validate_token()
        """
        _response = self._raw_client.validate_token(request_options=request_options)
        return _response.data

    def dev_login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LoginVerificationResponse:
        """
        Development login (bypass email verification).

        **WARNING:** This endpoint is only available in non-production environments
        or when explicitly enabled via configuration.

        It validates credentials and immediately returns authentication tokens
        without sending a verification code.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LoginVerificationResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos, DeviceInformation

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.dev_login(
            email_or_username="user@example.com",
            password="SecurePassword123!",
            device_information=DeviceInformation(
                device="Chrome Browser",
                device_id="device-123",
                operating_system="macOS",
                platform="web",
            ),
        )
        """
        _response = self._raw_client.dev_login(
            email_or_username=email_or_username,
            password=password,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    def google_authorize(
        self,
        *,
        redirect_uri: str,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GoogleAuthorizeResponse:
        """
        Generate Google OAuth authorization URL for Sign-In.

        Parameters
        ----------
        redirect_uri : str
            Frontend URL to redirect to after authentication

        organization_id : typing.Optional[str]
            Optional organization ID to associate user with

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GoogleAuthorizeResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.google_authorize(
            redirect_uri="https://app.aitronos.com/auth/callback",
        )
        """
        _response = self._raw_client.google_authorize(
            redirect_uri=redirect_uri, organization_id=organization_id, request_options=request_options
        )
        return _response.data

    def google_callback(
        self,
        *,
        code: str,
        state: str,
        error: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Handle Google OAuth callback and show success/error page.

        Parameters
        ----------
        code : str
            Authorization code from Google

        state : str
            CSRF state token

        error : typing.Optional[str]
            Error from Google

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.authentication.google_callback(
            code="code",
            state="state",
        )
        """
        _response = self._raw_client.google_callback(
            code=code, state=state, error=error, request_options=request_options
        )
        return _response.data


class AsyncAuthenticationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthenticationClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuthenticationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthenticationClient
        """
        return self._raw_client

    async def register(
        self,
        *,
        email: str,
        password: str,
        full_name: str,
        user_name: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterResponse:
        """
        Register a new user account with email verification.

        Creates a new user account, sends a 4-digit verification code via email,
        and creates an EmailVerification record. User must verify their email
        using the /verify endpoint to complete registration.

        Parameters
        ----------
        email : str
            User email address (primary identifier)

        password : str
            Password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        full_name : str
            User's full name

        user_name : typing.Optional[str]
            Preferred username (defaults to email prefix if omitted, min 3 chars)

        organization_id : typing.Optional[str]
            Organization UID to associate the user with

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos, DeviceInformation

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.register(
                email="user@company.com",
                password="SecurePassword123!",
                full_name="John Doe",
                user_name="johndoe",
                organization_id="org_12345678901234567890123456789012",
                device_information=DeviceInformation(
                    device="Chrome Browser",
                    device_id="device-123",
                    platform="web",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.register(
            email=email,
            password=password,
            full_name=full_name,
            user_name=user_name,
            organization_id=organization_id,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    async def login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LoginResponse:
        """
        Authenticate user credentials and initiate two-factor authentication (2FA) via email verification.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LoginResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos, DeviceInformation

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.login(
                email_or_username="user@example.com",
                password="SecurePassword123!",
                device_information=DeviceInformation(
                    device="Chrome Browser",
                    device_id="device-123",
                    operating_system="macOS",
                    platform="web",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.login(
            email_or_username=email_or_username,
            password=password,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    async def verify_email(
        self,
        *,
        email_key: str,
        verification_code: int,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LoginVerificationResponse:
        """
        Verify email verification code and return JWT tokens.

        Parameters
        ----------
        email_key : str
            Email verification key

        verification_code : int
            4-digit verification code

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LoginVerificationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos, DeviceInformation

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.verify_email(
                email_key="uuid-12345678-1234-1234-1234-123456789abc",
                verification_code=1234,
                device_information=DeviceInformation(
                    device_id="device-123",
                    platform="web",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.verify_email(
            email_key=email_key,
            verification_code=verification_code,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    async def refresh_token(
        self, *, refresh_token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RefreshTokenResponse:
        """
        Refresh an expired access token using a valid refresh token.

        Parameters
        ----------
        refresh_token : str
            Refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefreshTokenResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.refresh_token(
                refresh_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.refresh_token(refresh_token=refresh_token, request_options=request_options)
        return _response.data

    async def logout(
        self, *, refresh_token: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> LogoutResponse:
        """
        Logout the current user and invalidate their authentication tokens.

        Parameters
        ----------
        refresh_token : typing.Optional[str]
            Refresh token to invalidate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LogoutResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.logout(
                refresh_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.logout(refresh_token=refresh_token, request_options=request_options)
        return _response.data

    async def request_password_reset(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PasswordResetResponse:
        """
        Request a password reset for a user account.

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset/verify
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordResetResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.request_password_reset(
                email="user@example.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.request_password_reset(email=email, request_options=request_options)
        return _response.data

    async def verify_password_reset(
        self,
        *,
        email: str,
        verification_code: int,
        new_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PasswordResetVerifyResponse:
        """
        Verify password reset code and set new password.

        Validates the 4-digit code and updates the user's password.
        All existing refresh tokens are revoked, forcing re-authentication.

        **Security Features:**
        - Code expires after 5 minutes
        - One-time use codes (marked as used after verification)
        - Password strength validation
        - All sessions revoked on success

        **Errors:**
        - 404: User not found
        - 422: Invalid/expired code or weak password

        Parameters
        ----------
        email : str
            User email address

        verification_code : int
            4-digit verification code

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordResetVerifyResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.verify_password_reset(
                email="user@example.com",
                verification_code=1234,
                new_password="NewSecurePassword123!",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.verify_password_reset(
            email=email, verification_code=verification_code, new_password=new_password, request_options=request_options
        )
        return _response.data

    async def update_password(
        self,
        *,
        current_password: str,
        new_password: str,
        confirm_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PasswordUpdateResponse:
        """
        Update password for authenticated user.

        Requires current password verification and enforces password strength requirements.
        All other device sessions are revoked while keeping the current session active.

        **Security Features:**
        - Requires authentication (Bearer token)
        - Verifies current password before allowing change
        - Password strength validation
        - Keeps current session active
        - Revokes all other device sessions
        - Logs password change for audit

        **Password Requirements:**
        - Minimum 8 characters
        - At least one special character
        - Not in common weak passwords list

        **Errors:**
        - 401: Current password is incorrect
        - 404: User not found
        - 422: Passwords don't match or new password is weak

        Parameters
        ----------
        current_password : str
            Current password for verification

        new_password : str
            New password (min 8 chars with complexity, max 72 chars due to bcrypt limit)

        confirm_password : str
            Confirmation of new password (must match new_password)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordUpdateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.update_password(
                current_password="CurrentPassword123!",
                new_password="NewSecurePassword456!",
                confirm_password="NewSecurePassword456!",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_password(
            current_password=current_password,
            new_password=new_password,
            confirm_password=confirm_password,
            request_options=request_options,
        )
        return _response.data

    async def validate_email(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> EmailValidationResponse:
        """
        Validate email for registration.

        Checks if email is:
        - Valid format
        - Not already registered
        - Associated with an organization domain

        This endpoint is used during registration to provide real-time feedback
        about email validity and availability.

        **Returns:**
        - valid: Whether email format is valid
        - available: Whether email is available for registration
        - organization_id: Organization ID if email domain matches (optional)
        - organization_name: Organization name if email domain matches (optional)

        Parameters
        ----------
        email : str
            Email address to validate for signup eligibility

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailValidationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.validate_email(
                email="user@aitronos.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.validate_email(email=email, request_options=request_options)
        return _response.data

    async def resend_verification_code(
        self, *, email_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ResendCodeResponse:
        """
        Resend verification code.

        Generates and sends a new 4-digit verification code to the user's email.
        Works for login, registration, and password reset verifications.

        **Security Features:**
        - Validates verification record exists
        - Checks if verification already used
        - Generates new random code
        - Updates expiry time

        **Errors:**
        - 404: Verification record not found
        - 422: Verification already used
        - 500: Email sending failed

        Parameters
        ----------
        email_key : str
            Email verification key

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResendCodeResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.resend_verification_code(
                email_key="uuid-12345678-1234-1234-1234-123456789abc",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.resend_verification_code(
            email_key=email_key, request_options=request_options
        )
        return _response.data

    async def forgot_password(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> PasswordResetResponse:
        """
        Request a password reset (alias for /password/reset).

        Sends a 4-digit verification code to the user's email if the account exists.
        For security, always returns success regardless of whether the email exists.

        **Security Features:**
        - Generic response (prevents email enumeration)
        - Rate limiting (1 request per minute per email)
        - Time-limited codes (5 minutes expiry)
        - Automatic cleanup of expired codes

        **Flow:**
        1. User requests password reset with email
        2. If account exists, receives 4-digit code via email
        3. User submits code + new password to /password/reset
        4. All existing sessions are revoked on successful reset

        Parameters
        ----------
        email : str
            User email address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PasswordResetResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.forgot_password(
                email="user@example.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.forgot_password(email=email, request_options=request_options)
        return _response.data

    async def validate_token(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TokenValidationResponse:
        """
        Validate current authentication token.

        Used by frontend to check if the session is still valid.
        If the request reaches this endpoint (passes AuthContext dependency),
        the token is valid.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TokenValidationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.validate_token()


        asyncio.run(main())
        """
        _response = await self._raw_client.validate_token(request_options=request_options)
        return _response.data

    async def dev_login(
        self,
        *,
        email_or_username: str,
        password: str,
        device_information: typing.Optional[DeviceInformation] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LoginVerificationResponse:
        """
        Development login (bypass email verification).

        **WARNING:** This endpoint is only available in non-production environments
        or when explicitly enabled via configuration.

        It validates credentials and immediately returns authentication tokens
        without sending a verification code.

        Parameters
        ----------
        email_or_username : str
            Email address or username

        password : str
            User password

        device_information : typing.Optional[DeviceInformation]
            Device tracking information

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LoginVerificationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos, DeviceInformation

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.dev_login(
                email_or_username="user@example.com",
                password="SecurePassword123!",
                device_information=DeviceInformation(
                    device="Chrome Browser",
                    device_id="device-123",
                    operating_system="macOS",
                    platform="web",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.dev_login(
            email_or_username=email_or_username,
            password=password,
            device_information=device_information,
            request_options=request_options,
        )
        return _response.data

    async def google_authorize(
        self,
        *,
        redirect_uri: str,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GoogleAuthorizeResponse:
        """
        Generate Google OAuth authorization URL for Sign-In.

        Parameters
        ----------
        redirect_uri : str
            Frontend URL to redirect to after authentication

        organization_id : typing.Optional[str]
            Optional organization ID to associate user with

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GoogleAuthorizeResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.google_authorize(
                redirect_uri="https://app.aitronos.com/auth/callback",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.google_authorize(
            redirect_uri=redirect_uri, organization_id=organization_id, request_options=request_options
        )
        return _response.data

    async def google_callback(
        self,
        *,
        code: str,
        state: str,
        error: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Handle Google OAuth callback and show success/error page.

        Parameters
        ----------
        code : str
            Authorization code from Google

        state : str
            CSRF state token

        error : typing.Optional[str]
            Error from Google

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.authentication.google_callback(
                code="code",
                state="state",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.google_callback(
            code=code, state=state, error=error, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SpeakerAnalysis(UniversalBaseModel):
    """
    Speaker diarization configuration.
    """

    diarization_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable speaker identification
    """

    speakers_expected_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Expected number of speakers (1-10)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.add_to_vector_store_response import AddToVectorStoreResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMeetingBuddyVectorStoreRoutingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_meeting_buddy_session_to_vector_store(
        self, session_id: str, *, vector_store_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AddToVectorStoreResponse]:
        """
        Formats the session's transcript and intelligence as a searchable document and adds it to the specified vector store for embedding.

        Parameters
        ----------
        session_id : str

        vector_store_id : str
            Target vector store ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddToVectorStoreResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/meeting-buddy/sessions/{jsonable_encoder(session_id)}/add-to-vector-store",
            method="POST",
            json={
                "vector_store_id": vector_store_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddToVectorStoreResponse,
                    parse_obj_as(
                        type_=AddToVectorStoreResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMeetingBuddyVectorStoreRoutingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_meeting_buddy_session_to_vector_store(
        self, session_id: str, *, vector_store_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AddToVectorStoreResponse]:
        """
        Formats the session's transcript and intelligence as a searchable document and adds it to the specified vector store for embedding.

        Parameters
        ----------
        session_id : str

        vector_store_id : str
            Target vector store ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddToVectorStoreResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/meeting-buddy/sessions/{jsonable_encoder(session_id)}/add-to-vector-store",
            method="POST",
            json={
                "vector_store_id": vector_store_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddToVectorStoreResponse,
                    parse_obj_as(
                        type_=AddToVectorStoreResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SessionParticipantEvent(UniversalBaseModel):
    """
    A participant join/leave event for a session.
    """

    name: str
    event: str = pydantic.Field()
    """
    'joined' or 'left'
    """

    timestamp: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .sync_response_status import SyncResponseStatus


class SyncResponse(UniversalBaseModel):
    """
    Response for sync operations.
    """

    status: SyncResponseStatus = pydantic.Field()
    """
    Overall sync status
    """

    sync_id: str = pydantic.Field()
    """
    Unique sync identifier
    """

    processed: int = pydantic.Field()
    """
    Total operations processed
    """

    results: typing.Dict[str, int] = pydantic.Field()
    """
    Results breakdown by operation type
    """

    by_namespace: typing.Dict[str, typing.Dict[str, int]] = pydantic.Field()
    """
    Results grouped by namespace
    """

    processing_time_ms: int = pydantic.Field()
    """
    Processing time in milliseconds
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .create_flowplate_key_response import CreateFlowplateKeyResponse


class RefreshFlowplateKeyResponse(UniversalBaseModel):
    """
    Response schema for Flowplate key refresh (includes new raw key).
    """

    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the refresh was successful
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Action result message
    """

    new_key: CreateFlowplateKeyResponse = pydantic.Field()
    """
    The new key details (old key is now invalid)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from .http_client import AsyncHttpClient, HttpClient
from .logging import LogConfig, Logger


class BaseClientWrapper:
    def __init__(
        self,
        *,
        api_key: str,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        self._api_key = api_key
        self._token = token
        self._headers = headers
        self._base_url = base_url
        self._timeout = timeout
        self._logging = logging

    def get_headers(self) -> typing.Dict[str, str]:
        import platform

        headers: typing.Dict[str, str] = {
            "User-Agent": "aitronos/4.5.8",
            "X-Fern-Language": "Python",
            "X-Fern-Runtime": f"python/{platform.python_version()}",
            "X-Fern-Platform": f"{platform.system().lower()}/{platform.release()}",
            **(self.get_custom_headers() or {}),
        }
        headers["x-api-key"] = self._api_key
        token = self._get_token()
        if token is not None:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    def _get_token(self) -> typing.Optional[str]:
        if isinstance(self._token, str) or self._token is None:
            return self._token
        else:
            return self._token()

    def get_custom_headers(self) -> typing.Optional[typing.Dict[str, str]]:
        return self._headers

    def get_base_url(self) -> str:
        return self._base_url

    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        api_key: str,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(
            api_key=api_key, token=token, headers=headers, base_url=base_url, timeout=timeout, logging=logging
        )
        self.httpx_client = HttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
            logging_config=self._logging,
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        api_key: str,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        async_token: typing.Optional[typing.Callable[[], typing.Awaitable[str]]] = None,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(
            api_key=api_key, token=token, headers=headers, base_url=base_url, timeout=timeout, logging=logging
        )
        self._async_token = async_token
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
            async_base_headers=self.async_get_headers,
            logging_config=self._logging,
        )

    async def async_get_headers(self) -> typing.Dict[str, str]:
        headers = self.get_headers()
        if self._async_token is not None:
            token = await self._async_token()
            headers["Authorization"] = f"Bearer {token}"
        return headers



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class LoginResponse(UniversalBaseModel):
    """
    Response schema for login endpoint.
    """

    success: bool
    message: str
    email_key: str
    email: str
    requires_verification: bool
    next_step: str
    type: str = pydantic.Field()
    """
    Verification type: 'login', 'registration', or 'password_reset'
    """

    user: typing.Dict[str, typing.Any]
    device_info: typing.Optional[typing.Dict[str, typing.Any]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class LoginVerificationResponse(UniversalBaseModel):
    """
    Response schema for login verification endpoint.
    """

    refresh_token: str = pydantic.Field()
    """
    Refresh token
    """

    access_token: str
    token_type: str
    expires_in: int
    device_id: typing.Optional[str] = None
    user: typing.Dict[str, typing.Any]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
